@@ -21,7 +21,6 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
 #include <stdlib.h>
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
@@ -40,6 +39,7 @@ static void gda_default_provider_init       (GdaDefaultProvider *provider,
 					     GdaDefaultProviderClass *klass);
 static void gda_default_provider_finalize   (GObject *object);
 
+static const gchar *gda_default_provider_get_version (GdaServerProvider *provider);
 static gboolean gda_default_provider_open_connection (GdaServerProvider *provider,
 						      GdaConnection *cnc,
 						      GdaQuarkList *params,
@@ -47,6 +47,10 @@ static gboolean gda_default_provider_open_connection (GdaServerProvider *provide
 						      const gchar *password);
 static gboolean gda_default_provider_close_connection (GdaServerProvider *provider,
 						       GdaConnection *cnc);
+static const gchar *gda_default_provider_get_server_version (GdaServerProvider *provider,
+							     GdaConnection *cnc);
+static const gchar *gda_default_provider_get_database (GdaServerProvider *provider,
+						       GdaConnection *cnc);
 static gboolean gda_default_provider_create_database (GdaServerProvider *provider,
 						      GdaConnection *cnc,
 						      const gchar *name);
@@ -86,8 +90,11 @@ gda_default_provider_class_init (GdaDefaultProviderClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_default_provider_finalize;
+	provider_class->get_version = gda_default_provider_get_version;
 	provider_class->open_connection = gda_default_provider_open_connection;
 	provider_class->close_connection = gda_default_provider_close_connection;
+	provider_class->get_server_version = gda_default_provider_get_server_version;
+	provider_class->get_database = gda_default_provider_get_database;
 	provider_class->create_database = gda_default_provider_create_database;
 	provider_class->execute_command = gda_default_provider_execute_command;
 	provider_class->begin_transaction = gda_default_provider_begin_transaction;
@@ -149,6 +156,16 @@ gda_default_provider_new (void)
 	return GDA_SERVER_PROVIDER (provider);
 }
 
+/* get_version handler for the GdaDefaultProvider class */
+static const gchar *
+gda_default_provider_get_version (GdaServerProvider *provider)
+{
+	GdaDefaultProvider *dfprv = (GdaDefaultProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_DEFAULT_PROVIDER (dfprv), NULL);
+	return VERSION;
+}
+
 /* open_connection handler for the GdaDefaultProvider class */
 static gboolean
 gda_default_provider_open_connection (GdaServerProvider *provider,
@@ -208,10 +225,48 @@ gda_default_provider_close_connection (GdaServerProvider *provider,
 	return TRUE;
 }
 
+/* get_server_version handler for the GdaDefaultProvider class */
+static const gchar *
+gda_default_provider_get_server_version (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	GdaDefaultProvider *dfprv = (GdaDefaultProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_DEFAULT_PROVIDER (dfprv), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	return VERSION; // FIXME
+}
+
+/* get_database handler for the GdaDefaultProvider class */
+static const gchar *
+gda_default_provider_get_database (GdaServerProvider *provider, GdaConnection *cnc)
+{
+	GdaXmlDatabase *xmldb;
+	GdaDefaultProvider *dfprv = (GdaDefaultProvider *) provider;
+
+	g_return_val_if_fail (GDA_IS_DEFAULT_PROVIDER (dfprv), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	xmldb = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_DEFAULT_HANDLE);
+	if (!xmldb) {
+		gda_connection_add_error_string (cnc, _("Invalid internal handle"));
+		return NULL;
+	}
+
+	return gda_xml_database_get_name (xmldb);
+}
+
 static GList *
 process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 {
 	gchar **arr;
+	GdaXmlDatabase *xmldb;
+
+	xmldb = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_DEFAULT_HANDLE);
+	if (!xmldb) {
+		gda_connection_add_error_string (cnc, _("Invalid internal handle"));
+		return reclist;
+	}
 
 	/* parse SQL string, which can contain several commands, separated by ';' */
 	arr = g_strsplit (sql, ";", 0);
@@ -228,6 +283,42 @@ process_sql_commands (GList *reclist, GdaConnection *cnc, const gchar *sql)
 	return reclist;
 }
 
+static GList *
+process_table_commands (GList *reclist, GdaConnection *cnc, const gchar *str)
+{
+	gchar **arr;
+	GdaXmlDatabase *xmldb;
+
+	xmldb = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_DEFAULT_HANDLE);
+	if (!xmldb) {
+		gda_connection_add_error_string (cnc, _("Invalid internal handle"));
+		return reclist;
+	}
+
+	/* parse string, which can contain several table names */
+	arr = g_strsplit (str, ";", 0);
+	if (arr) {
+		gint n = 0;
+
+		while (arr[n]) {
+			GdaTable *table;
+
+			table = gda_xml_database_find_table (xmldb, arr[n]);
+			if (GDA_IS_TABLE (table))
+				reclist = g_list_append (reclist, table);
+			else {
+				// FIXME: what to do on errors?
+			}
+
+			n++;
+		}
+
+		g_strfreev (arr);
+	}
+
+	return reclist;
+}
+
 /* create_database handler for the GdaDefaultProvider class */
 static gboolean
 gda_default_provider_create_database (GdaServerProvider *provider,
@@ -269,6 +360,8 @@ gda_default_provider_execute_command (GdaServerProvider *provider,
 			/* FIXME: Implement */
 			return NULL;
 		case GDA_COMMAND_TYPE_TABLE:
+			reclist = process_table_commands (reclist, cnc,
+							  gda_command_get_text (cmd));
 			break;
 		case GDA_COMMAND_TYPE_INVALID:
 			return NULL;
@@ -339,7 +432,7 @@ gda_default_provider_supports (GdaServerProvider *provider,
 			       GdaConnectionFeature feature)
 {
 	if (feature == GDA_CONNECTION_FEATURE_TRANSACTIONS
-	    && feature == GDA_CONNECTION_FEATURE_XML_QUERIES)
+	    || feature == GDA_CONNECTION_FEATURE_XML_QUERIES)
 		return TRUE;
 
 	return FALSE;
@@ -459,10 +552,7 @@ get_databases (GdaConnection *cnc, GdaXmlDatabase *xmldb)
 	const gchar *dbname;
 
 	recset = GDA_DATA_MODEL_ARRAY (gda_data_model_array_new (1));
-	//gda_server_recordset_model_set_field_defined_size (recset, 0, 256);
 	gda_data_model_set_column_title (GDA_DATA_MODEL (recset), 0, _("Name"));
-	//gda_server_recordset_model_set_field_scale (recset, 0, 0);
-	//gda_server_recordset_model_set_field_gdatype (recset, 0, GDA_TYPE_STRING);
 
 	dbname = gda_xml_database_get_name (xmldb);
 	if (dbname != NULL)
