@@ -2,7 +2,7 @@
  * Copyright (C) 2002 The GNOME Foundation
  *
  * AUTHORS:
- *         Holger Thon <holger.thon@gnome-db-org>
+ *         Holger Thon <holger.thon@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,9 +19,12 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <config.h>
+#if defined(HAVE_CONFIG_H)
+#endif
+
 #include <libgda/gda-data-model-array.h>
 #include <libgda/gda-intl.h>
+#include <libgda/gda-log.h>
 #include <glib.h>
 #include <stdlib.h>
 #include <tds.h>
@@ -94,16 +97,34 @@ static GList* gda_freetds_provider_process_sql_commands(GList         *reclist,
                                                         GdaConnection *cnc,
                                                         const gchar   *sql);
 
-static gboolean tds_cbs_initialized = FALSE;
-extern int (*g_tds_msg_handler)();
-extern int (*g_tds_err_handler)();
+static gchar *gda_freetds_get_stringresult_of_query (GdaConnection *cnc, 
+                                                     const gchar *sql,
+                                                     const gint col,
+                                                     const gint row);
+
+#ifdef HAVE_FREETDS_VER0_5X
+  static gboolean tds_cbs_initialized = FALSE;
+  extern int (*g_tds_msg_handler)();
+  extern int (*g_tds_err_handler)();
+#endif
 
 static void gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc);
 
+static gboolean gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql);
+
 static int gda_freetds_provider_tds_handle_message (void *aStruct,
                                                     const gboolean is_err_msg);
-static int gda_freetds_provider_tds_handle_info_msg (void *aStruct);
-static int gda_freetds_provider_tds_handle_err_msg (void *aStruct);
+#ifdef HAVE_FREETDS_VER0_6X
+  static int gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *,
+                                                       TDSSOCKET *,
+                                                       TDSMSGINFO *);
+  static int gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *,
+                                                      TDSSOCKET *,
+                                                      TDSMSGINFO *);
+#else
+  static int gda_freetds_provider_tds_handle_info_msg (void *aStruct);
+  static int gda_freetds_provider_tds_handle_err_msg (void *aStruct);
+#endif
 
 static gboolean
 gda_freetds_provider_open_connection (GdaServerProvider *provider,
@@ -141,7 +162,7 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	t_host = gda_quark_list_find (params, "HOST");
 	t_hostaddr = gda_quark_list_find (params, "HOSTADDR");
 	t_options = gda_quark_list_find (params, "OPTIONS");
-	t_user = gda_quark_list_find (params, "USERNAME");
+	t_user = gda_quark_list_find (params, "USER");
 	t_password = gda_quark_list_find (params, "PASSWORD");
 	t_port = gda_quark_list_find (params, "PORT");
 	
@@ -213,18 +234,41 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	tds_set_user(tds_cnc->login, (char *) t_user);
 	tds_set_passwd(tds_cnc->login, (char *) t_password);
 	tds_set_app(tds_cnc->login, "libgda");
+	
 	if (t_hostaddr)
 		tds_set_host(tds_cnc->login, (char *) t_hostaddr);
+	
 	tds_set_library(tds_cnc->login, "TDS-Library");
 	if (t_host)
 		tds_set_server(tds_cnc->login, (char *) t_host);
-	tds_set_charset(tds_cnc->login, "iso-1");
+	
+	tds_set_charset(tds_cnc->login, "iso_1");
 	tds_set_language(tds_cnc->login, "us_english");
 	tds_set_packet(tds_cnc->login, 512);
-	
-	// establish connection
+
+	// Version 0.60 api uses context additionaly
+#ifdef HAVE_FREETDS_VER0_6X
+	tds_cnc->ctx = tds_alloc_context();
+	if (! tds_cnc->ctx) {
+		gda_log_error (_("Allocating tds context failed."));
+		gda_freetds_free_connection_data (tds_cnc);
+		error = gda_freetds_make_error(NULL, _("Allocating tds context failed."));
+		gda_connection_add_error(cnc, error);
+		return FALSE;
+	}
+	// Initialize callbacks which are now in context struct
+	tds_cnc->ctx->msg_handler = gda_freetds_provider_tds_handle_info_msg;
+	tds_cnc->ctx->err_handler = gda_freetds_provider_tds_handle_err_msg;
+#endif
+
+	// establish connection; change in 0.6x api
+#ifndef HAVE_FREETDS_VER0_6X
 	tds_cnc->tds = tds_connect(tds_cnc->login, NULL);
+#else
+	tds_cnc->tds = tds_connect(tds_cnc->login, tds_cnc->ctx, NULL);
+#endif
 	if (! tds_cnc->tds) {
+		gda_log_error (_("Establishing connection failed."));
 		gda_freetds_free_connection_data (tds_cnc);
 		error = gda_freetds_make_error(NULL, _("Establishing connection failed."));
 		gda_connection_add_error(cnc, error);
@@ -232,22 +276,43 @@ gda_freetds_provider_open_connection (GdaServerProvider *provider,
 	}
 
 	// try to receive connection info for sanity check
+#ifndef HAVE_FREETDS_VER0_6X
 	tds_cnc->config = tds_get_config(tds_cnc->tds, tds_cnc->login);
+#else
+	tds_cnc->config = tds_get_config(tds_cnc->tds, tds_cnc->login, tds_cnc->ctx->locale);
+#endif
 	if (! tds_cnc->config) {
+		gda_log_error (_("Failed getting connection info."));	
 		error = gda_freetds_make_error (tds_cnc->tds, _("Failed getting connection info."));
 		gda_connection_add_error (cnc, error);
 
-		tds_free_socket(tds_cnc->tds);
-		tds_cnc->tds = NULL;
-		tds_free_login(tds_cnc->login);
-		tds_cnc->login = NULL;
 		gda_freetds_free_connection_data (tds_cnc);
 		return FALSE;
 	}
 
-	tds_cnc->rc = TDS_SUCCEED;
+	// Pass cnc pointer to tds (user data ptr) for reuse in callbacks
 	tds_set_parent (tds_cnc->tds, (void *) cnc);
+	// set cnc object handle; 
+	// thus make calls to e.g. gda_freetds_execute_cmd possible
 	g_object_set_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE, tds_cnc);
+	
+	if ((t_database != NULL)
+	    && (gda_freetds_provider_change_database(provider,
+	                                             cnc, t_database
+	                                            ) != TRUE)) {
+		g_object_set_data (G_OBJECT (cnc),
+		                   OBJECT_DATA_FREETDS_HANDLE,
+				   NULL);
+		gda_freetds_free_connection_data (tds_cnc);
+		return FALSE;
+	}
+
+	// fixme: these have been removed in debian sid
+	// 
+	// tds_cnc->is_sybase = TDS_IS_SYBASE (tds_cnc->tds);
+	// tds_cnc->srv_ver = tds_cnc->tds->product_version;
+	
+	tds_cnc->rc = TDS_SUCCEED;
 
 	return TRUE;
 }
@@ -259,15 +324,33 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 
 	g_return_if_fail (tds_cnc != NULL);
 
+	if (tds_cnc->server_id) {
+		g_free (tds_cnc->server_id);
+		tds_cnc->server_id = NULL;
+	}
+	if (tds_cnc->database) {
+		g_free (tds_cnc->database);
+		tds_cnc->database = NULL;
+	}
 	if (tds_cnc->config) {
 		tds_free_config(tds_cnc->config);
 		tds_cnc->config = NULL;
 	}
 	if (tds_cnc->tds) {
+		// clear tds user data pointer
 		tds_set_parent (tds_cnc->tds, NULL);
 		tds_free_socket (tds_cnc->tds);
 		tds_cnc->tds = NULL;
 	}
+#ifdef HAVE_FREETDS_VER0_6X
+	if (tds_cnc->ctx) {
+		// Clear callback handler
+		tds_cnc->ctx->msg_handler = NULL;
+		tds_cnc->ctx->err_handler = NULL;
+		tds_free_context (tds_cnc->ctx);
+		tds_cnc->ctx = NULL;
+	}
+#endif
 	if (tds_cnc->login) {
 		tds_free_login(tds_cnc->login);
 		tds_cnc->login = NULL;
@@ -282,22 +365,29 @@ gda_freetds_free_connection_data (GdaFreeTDSConnectionData *tds_cnc)
 
 			g_ptr_array_remove_index (tds_cnc->err_arr, 0);
 		}
+			
 		g_ptr_array_free (tds_cnc->err_arr, TRUE);
 		tds_cnc->err_arr = NULL;
 	}
+	
 	if (tds_cnc->msg_arr) {
 		while (tds_cnc->msg_arr->len > 0) {
 			msg = (GdaFreeTDSMessage *) g_ptr_array_index (tds_cnc->msg_arr, 0);
+					
 			if (msg != NULL) {
 				gda_freetds_message_free (msg);
 			}
 
 			g_ptr_array_remove_index (tds_cnc->msg_arr, 0);
 		}
+			
 		g_ptr_array_free (tds_cnc->msg_arr, TRUE);
 		tds_cnc->msg_arr = NULL;
 	}
 	
+	tds_cnc->is_sybase = FALSE;
+	tds_cnc->srv_ver = 0;
+		
 	g_free(tds_cnc);
 	tds_cnc = NULL;
 }
@@ -316,20 +406,6 @@ gda_freetds_provider_close_connection (GdaServerProvider *provider,
 	if (! tds_cnc)
 		return FALSE;
 
-	if (tds_cnc->config) {
-		tds_free_config(tds_cnc->config);
-		tds_cnc->config = NULL;
-	}
-	if (tds_cnc->tds) {
-		tds_set_parent (tds_cnc->tds, NULL);
-		tds_free_socket (tds_cnc->tds);
-		tds_cnc->tds = NULL;
-	}
-	if (tds_cnc->login) {
-		tds_free_login(tds_cnc->login);
-		tds_cnc->login = NULL;
-	}
-
 	gda_freetds_free_connection_data (tds_cnc);
 
 	return TRUE;
@@ -347,14 +423,15 @@ static const gchar
 
 	tds_cnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE);
 	g_return_val_if_fail (tds_cnc != NULL, NULL);
-	
-	if (tds_cnc->config) {
-		tds_free_config(tds_cnc->config);
-		tds_cnc->config = NULL;
+	g_return_val_if_fail (tds_cnc->tds != NULL, NULL);
+
+	if (tds_cnc->database) {
+		g_free (tds_cnc->database);
 	}
-	tds_cnc->config = tds_get_config(tds_cnc->tds, tds_cnc->login);
-	
-	return tds_cnc->config->database;
+	tds_cnc->database = gda_freetds_get_stringresult_of_query (cnc,
+	                       TDS_QUERY_CURRENT_DATABASE, 0, 0);
+
+	return (const gchar*) tds_cnc->database;
 }
 
 static gboolean
@@ -362,12 +439,18 @@ gda_freetds_provider_change_database (GdaServerProvider *provider,
                                       GdaConnection *cnc,
                                       const gchar *name)
 {
+	gchar *sql_cmd = NULL;
+	gboolean ret = FALSE;
 	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
 
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), FALSE);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
 	
-	return FALSE;
+	sql_cmd = g_strdup_printf("USE %s", name);
+	ret = gda_freetds_execute_cmd(cnc, sql_cmd);
+	g_free(sql_cmd);
+	
+	return ret;
 }
 
 static gboolean
@@ -426,6 +509,12 @@ static GList
 			g_free(query);
 			query = NULL;
 			break;
+		case GDA_COMMAND_TYPE_XML:
+		case GDA_COMMAND_TYPE_PROCEDURE:
+		case GDA_COMMAND_TYPE_SCHEMA:
+		case GDA_COMMAND_TYPE_INVALID:
+			return reclist;
+			break;
 	}
 
 	return reclist;
@@ -485,6 +574,16 @@ gda_freetds_provider_supports (GdaServerProvider *provider,
 		case GDA_CONNECTION_FEATURE_USERS:
 		case GDA_CONNECTION_FEATURE_VIEWS:
 			return TRUE;
+
+	//FIXME: Implement missing 
+		case GDA_CONNECTION_FEATURE_AGGREGATES:
+		case GDA_CONNECTION_FEATURE_INDEXES:
+		case GDA_CONNECTION_FEATURE_INHERITANCE:
+		case GDA_CONNECTION_FEATURE_SEQUENCES:
+		case GDA_CONNECTION_FEATURE_TRANSACTIONS:
+		case GDA_CONNECTION_FEATURE_TRIGGERS:
+		case GDA_CONNECTION_FEATURE_XML_QUERIES:
+			return FALSE;
 	}
 	
 	return FALSE;
@@ -496,14 +595,29 @@ static const gchar
 {
 	GdaFreeTDSProvider *tds_prov = (GdaFreeTDSProvider *) provider;
 	GdaFreeTDSConnectionData *tds_cnc = NULL;
+	GdaDataModel *model = NULL;
 	
 	g_return_val_if_fail (GDA_IS_FREETDS_PROVIDER (tds_prov), NULL);
 	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
 	tds_cnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE);
 	g_return_val_if_fail (tds_cnc != NULL, NULL);
 
-	// FIXME:
-	return NULL;
+	if (!tds_cnc->server_id) {
+		model = gda_freetds_execute_query (cnc, TDS_QUERY_SERVER_VERSION);
+		if (model) {
+			if ((gda_data_model_get_n_columns (model) == 1)
+			    && (gda_data_model_get_n_rows (model) == 1)) {
+				GdaValue *value;
+
+				value = (GdaValue *) gda_data_model_get_value_at
+				                             (model, 0, 0);
+				tds_cnc->server_id = gda_value_stringify ((GdaValue *) value);
+			}
+			g_object_unref (model);
+		}
+	}
+
+	return (const gchar *) tds_cnc->server_id;
 }
 
 static const gchar
@@ -516,6 +630,62 @@ static const gchar
 	return VERSION;
 }
 
+static GdaDataModel
+*gda_freetds_provider_get_types (GdaConnection    *cnc,
+                                 GdaParameterList *params)
+{
+	GdaDataModel *model = NULL;
+	TDSCOLINFO col;
+	GdaValueType gda_type;
+	GdaValue     *value = NULL;
+	gint i = 1;
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	model = gda_freetds_execute_query (cnc, TDS_SCHEMA_TYPES);
+	TDS_FIXMODEL_SCHEMA_TYPES (model);
+
+	memset (&col, 0, sizeof (col));
+	
+	if (model) {
+		for (i = 0; i < gda_data_model_get_n_rows (model); i++) {
+			GdaRow *row = (GdaRow *) gda_data_model_get_row (model,
+			                                                 i);
+			
+			// first fix gda_type
+			if (row) {
+				value = gda_row_get_value (row, 2);
+				
+				if (gda_value_get_type (value) == GDA_VALUE_TYPE_INTEGER) {
+					col.column_size = gda_value_get_integer (value);
+				} else {
+					col.column_size = 0;
+				}
+				value = gda_row_get_value (row, 3);
+				if (gda_value_get_type (value) != GDA_VALUE_TYPE_TINYINT) {
+					col.column_type = SYBVARIANT;
+				} else {
+					col.column_type = gda_value_get_tinyint (value);
+				}
+				
+				gda_type = gda_freetds_get_value_type (&col);
+				// col 3: type -> clear and set
+				//   step 1: make sure value is cleared
+				gda_value_set_null (value);
+				memset (value, 0, sizeof(GdaValue));
+				//   step 2: set type of gdavalue
+				value->type = GDA_VALUE_TYPE_TYPE;
+				value->value.v_type = gda_type;
+				
+				// col 2: comment -> clear
+				value = gda_row_get_value (row, 2);
+				gda_value_set_string (value, "");
+			}
+		}
+	}
+
+	return model;
+}
+
 static GdaDataModel
 *gda_freetds_provider_get_schema (GdaServerProvider *provider,                                                    GdaConnection *cnc,
                                   GdaConnectionSchema schema,
@@ -531,47 +701,97 @@ static GdaDataModel
 	switch (schema) {
 		case GDA_CONNECTION_SCHEMA_DATABASES:
 			return gda_freetds_get_databases (cnc, params);
+			break;
 		case GDA_CONNECTION_SCHEMA_FIELDS:
 			return gda_freetds_get_fields (cnc, params);
+			break;
 		case GDA_CONNECTION_SCHEMA_PROCEDURES:
 			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_PROCEDURES);
-			g_free (query);
-			query = NULL;
-			if (recset != NULL)
-				gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
-				                                 0, _("Procedures"));
+			TDS_FIXMODEL_SCHEMA_PROCEDURES (recset)
+			
 			return recset;
+			break;
 		case GDA_CONNECTION_SCHEMA_TABLES:
 			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_TABLES);
-			g_free (query);
-			query = NULL;
-			if (recset != NULL)
-				gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
-				                                 0, _("Tables"));
+			TDS_FIXMODEL_SCHEMA_TABLES (recset)
+			
 			return recset;
+			break;
 		case GDA_CONNECTION_SCHEMA_TYPES:
-			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_TYPES);
-			if (recset != NULL)
-				gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
-				                                 0, _("Types"));
+			recset = gda_freetds_provider_get_types (cnc, params);
+
 			return recset;
+			break;
 		case GDA_CONNECTION_SCHEMA_USERS:
-			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_TYPES);
-			if (recset != NULL)
-				gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
-				                                 0, _("Users"));
+			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_USERS);
+			TDS_FIXMODEL_SCHEMA_USERS (recset)
+				
 			return recset;
+			break;
 		case GDA_CONNECTION_SCHEMA_VIEWS:
 			recset = gda_freetds_execute_query (cnc, TDS_SCHEMA_VIEWS);
-			if (recset != NULL)
-				gda_data_model_set_column_title (GDA_DATA_MODEL (recset),
-				                                 0, _("Views"));
+			TDS_FIXMODEL_SCHEMA_VIEWS (recset)
+
 			return recset;
+			break;
+
+		// FIXME: Implement aggregates, indexes, sequences and triggers
+		case GDA_CONNECTION_SCHEMA_AGGREGATES:
+		case GDA_CONNECTION_SCHEMA_INDEXES:
+		case GDA_CONNECTION_SCHEMA_PARENT_TABLES:
+		case GDA_CONNECTION_SCHEMA_SEQUENCES:
+		case GDA_CONNECTION_SCHEMA_TRIGGERS:
+			return NULL;
+			break;
 	}
 	
 	return NULL;
 }
 
+static gboolean
+gda_freetds_execute_cmd (GdaConnection *cnc, const gchar *sql)
+{
+	GdaFreeTDSConnectionData *tds_cnc;
+	GdaError *error;
+
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
+	g_return_val_if_fail (sql != NULL, FALSE);
+	tds_cnc = g_object_get_data (G_OBJECT (cnc), OBJECT_DATA_FREETDS_HANDLE);
+
+	g_return_val_if_fail (tds_cnc != NULL, FALSE);
+	g_return_val_if_fail (tds_cnc->tds != NULL, FALSE);
+
+	tds_cnc->rc = tds_submit_query (tds_cnc->tds, (char *) sql);
+	if (tds_cnc->rc != TDS_SUCCEED) {
+		gda_log_error (_("Query did not succeed in execute_cmd()."));
+		error = gda_freetds_make_error (tds_cnc->tds, _("Query did not succeed in execute_cmd()."));
+		gda_connection_add_error (cnc, error);
+		return FALSE;
+	}
+
+	// there should not be any result tokens
+	while ((tds_cnc->rc = tds_process_result_tokens (tds_cnc->tds)) 
+	       == TDS_SUCCEED) {
+		if (tds_cnc->tds->res_info->rows_exist) {
+			gda_log_error (_("Unexpected result tokens in execute_cmd()."));
+			error = gda_freetds_make_error (tds_cnc->tds,
+			                                _("Unexpected result tokens in execute_cmd()."));
+			gda_connection_add_error (cnc, error);
+			return FALSE;
+		}
+	}
+	
+	if ((tds_cnc->rc != TDS_FAIL) && (tds_cnc->rc != TDS_NO_MORE_RESULTS)) {
+		error = gda_freetds_make_error (tds_cnc->tds,
+		                                _("Unexpected return in execute_cmd()."));
+		gda_log_error (_("Unexpected return in execute_cmd()."));
+		gda_connection_add_error (cnc, error);
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static GdaDataModel *
 gda_freetds_execute_query (GdaConnection *cnc, const gchar* sql)
 {
@@ -600,6 +820,36 @@ gda_freetds_execute_query (GdaConnection *cnc, const gchar* sql)
 	return recset;
 }
 
+// make sure to g_free() result after use
+static gchar *
+gda_freetds_get_stringresult_of_query (GdaConnection *cnc, 
+                                       const gchar *sql,
+                                       const gint col,
+                                       const gint row)
+{
+	GdaDataModel    *model = NULL;
+	GdaValue        *value = NULL;
+	gchar           *ret = NULL;
+
+	// GDA_IS_CONNECTION (cnc) validation in function call
+	model = gda_freetds_execute_query (cnc, sql);
+	
+	if (model) {
+		value = (GdaValue *) gda_data_model_get_value_at (model,
+		                                                  col, row);
+
+		if ((value != NULL)
+		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_NULL)
+		    && (gda_value_get_type (value) != GDA_VALUE_TYPE_UNKNOWN)) {
+			ret = (gchar *) gda_value_stringify (value);
+		}
+		
+		g_object_unref (model);
+	}
+
+	return ret;
+}
+
 static GdaDataModel *
 gda_freetds_get_databases (GdaConnection *cnc, GdaParameterList *params)
 {
@@ -614,6 +864,7 @@ gda_freetds_get_databases (GdaConnection *cnc, GdaParameterList *params)
 
 	recset = GDA_FREETDS_RECORDSET (reclist->data);
 	g_list_free (reclist);
+	TDS_FIXMODEL_SCHEMA_DATABASES(recset)
 
 	return GDA_DATA_MODEL (recset);
 }
@@ -641,24 +892,7 @@ gda_freetds_get_fields (GdaConnection *cnc, GdaParameterList *params)
 	query = NULL;
 
 	if (GDA_IS_FREETDS_RECORDSET (recset)) {
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  0,
-		                                 _("Field Name"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  1,
-		                                 _("Data Type"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  2,
-		                                 _("Size"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  3,
-		                                 _("Precision"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  4,
-		                                 _("Scale"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  5,
-		                                 _("Identity"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  6,
-		                                 _("Nullable"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  7,
-		                                 _("Domain"));
-		gda_data_model_set_column_title (GDA_DATA_MODEL (recset),  8,
-		                                 _("Printfmt"));
+		TDS_FIXMODEL_SCHEMA_FIELDS (recset)
 	}
 
 	return recset;
@@ -732,12 +966,14 @@ gda_freetds_provider_class_init (GdaFreeTDSProviderClass *klass)
 	provider_class->supports = gda_freetds_provider_supports;
 	provider_class->get_schema = gda_freetds_provider_get_schema;
 
+#ifdef HAVE_FREETDS_VER0_5X
 	if (tds_cbs_initialized == FALSE) {
 		tds_cbs_initialized = TRUE;
 
 		g_tds_msg_handler = gda_freetds_provider_tds_handle_info_msg;
 		g_tds_err_handler = gda_freetds_provider_tds_handle_err_msg;
 	}
+#endif
 }
 
 static void
@@ -753,14 +989,18 @@ gda_freetds_provider_finalize (GObject *object)
 
 	g_return_if_fail (GDA_IS_FREETDS_PROVIDER (provider));
 
+#ifdef HAVE_FREETDS_VER0_5X
 	tds_cbs_initialized = FALSE;
 	g_tds_msg_handler = NULL;
 	g_tds_err_handler = NULL;
+#endif
 
 	/* chain to parent class */
 	parent_class->finalize (object);
 }
 
+// FIXME: rewrite this function to suit new callback parameters
+//        (0.6x code)
 static int gda_freetds_provider_tds_handle_message (void *aStruct,
                                                     const gboolean is_err_msg)
 {
@@ -775,7 +1015,8 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 	cnc = (GdaConnection *) tds_get_parent (tds);
 	
 	// what if we cannot determine where the message belongs to?
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), TDS_SUCCEED);
+	g_return_val_if_fail (((GDA_IS_CONNECTION (cnc)) || (cnc == NULL)),
+	                      TDS_SUCCEED);
 
 	msg = g_strdup_printf(_("Msg %d, Level %d, State %d, Server %s, Line %d\n%s\n"),
 	                      tds->msg_info->msg_number,
@@ -787,18 +1028,24 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 
 	// if errormessage, proceed
 	if (is_err_msg == TRUE) {
-		error = gda_error_new ();
-		gda_error_set_description (error, msg);
-		gda_error_set_number (error, tds->msg_info->msg_number);
-		gda_error_set_source (error, "gda-freetds");
-		if (tds->msg_info->sql_state != NULL) {
-			gda_error_set_sqlstate (error,
-			                        tds->msg_info->sql_state);
-		} else {
-			gda_error_set_sqlstate (error, _("Not available"));
-		}
+		if (cnc != NULL) {
+			error = gda_error_new ();
+			gda_error_set_description (error, msg);
+			gda_error_set_number (error, tds->msg_info->msg_number);
+			gda_error_set_source (error, "gda-freetds");
+			if (tds->msg_info->sql_state != NULL) {
+				gda_error_set_sqlstate (error,
+				                        tds->msg_info->sql_state);
+			} else {
+				gda_error_set_sqlstate (error, _("Not available"));
+			}
 
-		gda_connection_add_error (cnc, error);
+			gda_connection_add_error (cnc, error);
+		} else {
+			gda_log_error (msg);
+		}
+	} else {
+		gda_log_message (msg);
 	}
 	
 	if (msg) {
@@ -809,16 +1056,37 @@ static int gda_freetds_provider_tds_handle_message (void *aStruct,
 	return TDS_SUCCEED;
 }
 
-static int gda_freetds_provider_tds_handle_info_msg (void *aStruct)
+#ifdef HAVE_FREETDS_VER0_6X
+// FIXME: rewrite tds_handle_message as well/use new parameters here
+static int
+gda_freetds_provider_tds_handle_info_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
+                                          TDSMSGINFO *msg)
+{
+	return gda_freetds_provider_tds_handle_message ((void *) tds, FALSE);
+}
+#else
+static int
+gda_freetds_provider_tds_handle_info_msg (void *aStruct)
 {
 	return gda_freetds_provider_tds_handle_message (aStruct, FALSE);
 }
+#endif
 
-static int gda_freetds_provider_tds_handle_err_msg (void *aStruct)
+#ifdef HAVE_FREETDS_VER0_6X
+// FIXME: rewrite tds_handle_message as well/use new parameters here
+static int
+gda_freetds_provider_tds_handle_err_msg (TDSCONTEXT *ctx, TDSSOCKET *tds,
+                                         TDSMSGINFO *msg)
+{
+	return gda_freetds_provider_tds_handle_message ((void *) tds, TRUE);
+}
+#else
+static int
+gda_freetds_provider_tds_handle_err_msg (void *aStruct)
 {
 	return gda_freetds_provider_tds_handle_message (aStruct, TRUE);
 }
-
+#endif
 
 /////////////////////////////////////////////////////////////////////////////
 // Public functions                                                       
@@ -852,5 +1120,9 @@ gda_freetds_provider_new (void)
 	GdaFreeTDSProvider *provider;
 
 	provider = g_object_new (gda_freetds_provider_get_type (), NULL);
+
+//	if (! gda_log_is_enabled())
+//		gda_log_enable();
+	
 	return GDA_SERVER_PROVIDER (provider);
 }
