@@ -20,10 +20,12 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <config.h>
+#include <string.h>
 #include <libgda/gda-intl.h>
 #include "gda-test.h"
 #include "postgres-test.h"
+#include "freetds-test.h"
+#include "sybase-test.h"
 
 /* Shows the object schemas */
 static void
@@ -70,10 +72,11 @@ open_connection (GdaClient *client,
 	GdaConnection *cnc;
 	gboolean res;
 	GdaDataSourceInfo *dsn_info;
+	GdaTransaction *xaction;
 
 	g_return_if_fail (GDA_IS_CLIENT (client));
 
-	cnc = gda_client_open_connection (client, name, username, password);
+	cnc = gda_client_open_connection (client, name, username, password, 0);
 	if (!GDA_IS_CONNECTION (cnc)) {
 		g_print (_("** ERROR: could not open connection to %s\n"), name);
 		return;
@@ -87,13 +90,16 @@ open_connection (GdaClient *client,
 	g_print (_("\t\tIndexes: %s\n"),
 		 gda_connection_supports (cnc, GDA_CONNECTION_FEATURE_INDEXES) ?
 		 _("Supported") : _("Not supported"));
+	g_print (_("\t\tNamespaces: %s\n"),
+		 gda_connection_supports (cnc, GDA_CONNECTION_FEATURE_NAMESPACES) ?
+		 _("Supported") : _("Not supported"));
 	g_print (_("\t\tProcedures: %s\n"),
 		 gda_connection_supports (cnc, GDA_CONNECTION_FEATURE_PROCEDURES) ?
 		 _("Supported") : _("Not supported"));
 	g_print (_("\t\tSequences: %s\n"),
 		 gda_connection_supports (cnc, GDA_CONNECTION_FEATURE_SEQUENCES) ?
 		 _("Supported") : _("Not supported"));
-	g_print (_("\t\tSQL language: %s\n"),
+	g_print (_("\t\tSQL: %s\n"),
 		 gda_connection_supports (cnc, GDA_CONNECTION_FEATURE_SQL) ?
 		 _("Supported") : _("Not supported"));
 	g_print (_("\t\tTransactions: %s\n"),
@@ -113,6 +119,9 @@ open_connection (GdaClient *client,
 		 _("Supported") : _("Not supported"));
 
 	/* show connection schemas */
+	g_print (_("\n\tProvider reports version: %s\n\n"),
+			gda_connection_get_server_version (cnc));
+	show_schema (cnc, GDA_CONNECTION_SCHEMA_NAMESPACES, _("Namespaces"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_DATABASES, _("Databases"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_PROCEDURES, _("Stored procedures"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_TABLES, _("Connection Tables"));
@@ -123,28 +132,37 @@ open_connection (GdaClient *client,
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_USERS, _("Users"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_SEQUENCES, _("Sequences"));
 	show_schema (cnc, GDA_CONNECTION_SCHEMA_TRIGGERS, _("Triggers"));
+	show_schema (cnc, GDA_CONNECTION_SCHEMA_LANGUAGES, _("Languages"));
 
 	/* test transactions */
+	xaction = gda_transaction_new (NULL);
+
 	g_print (_("\tStarting transaction..."));
-	res = gda_connection_begin_transaction (cnc, NULL);
+	res = gda_connection_begin_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tFinishing transaction..."));
-	res = gda_connection_commit_transaction (cnc, NULL);
+	res = gda_connection_commit_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tStarting transaction..."));
-	res = gda_connection_begin_transaction (cnc, NULL);
+	res = gda_connection_begin_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
 	g_print (_("\tRolling back transaction..."));
-	res = gda_connection_rollback_transaction (cnc, NULL);
+	res = gda_connection_rollback_transaction (cnc, xaction);
 	g_print ("%s\n", res ? "OK" : _("Error"));
 
+	g_object_unref (G_OBJECT (xaction));
+
 	dsn_info = gda_config_find_data_source (name);
 	/* Postgres own tests */
 	if (!strcmp (dsn_info->provider, POSTGRES_PROVIDER_NAME))
 		do_postgres_test (cnc);
+	if (!strcmp (dsn_info->provider, FREETDS_PROVIDER_NAME))
+		do_freetds_test (cnc);
+	if (!strcmp (dsn_info->provider, SYBASE_PROVIDER_NAME))
+		do_sybase_test (cnc);
 	gda_config_free_data_source_info (dsn_info);
 
 	/* close the connection */
@@ -173,7 +191,8 @@ test_client (void)
 		}
 
 		g_print (_(" Data source = %s, User = %s\n"), info->name, info->username);
-		open_connection (client, info->name, info->username, "");
+		open_connection (client, info->name, info->username,
+		                 (info->password) ? info->password : "");
 	}
 
 	/* free memory */
